<?php

namespace Drupal\erl_restriction\Plugin\Field\FieldType;

use Drupal\entity_reference_layout\Plugin\Field\FieldType\EntityReferenceLayoutRevisioned;
use Drupal\Core\Form\FormStateInterface;

/**
 * Entity Reference Layout Revisioned and restrictions field type.
 *
 * @FieldType(
 *   id = "erl_restriction",
 *   label = @Translation("Entity Reference with Layout and restriction"),
 *   description = @Translation("An entity field with layouts containing revisioned entity references."),
 *   category = @Translation("Reference revisions"),
 *   default_widget = "erl_restriction_widget",
 *   default_formatter = "entity_reference_layout",
 *   list_class = "\Drupal\entity_reference_layout\EntityReferenceLayoutRevisionsFieldItemList",
 * )
 */
class ErlRestrictionType extends EntityReferenceLayoutRevisioned {

  /**
   * Manipulate field data to be saved as configuration.
   */
  public static function fieldSettingsToConfigData(array $settings) {
    $settings = parent::fieldSettingsToConfigData($settings);

    if (!isset($settings['handler_settings']['restrictions'])) {
      return $settings;
    }

    // Create array with only selected paragraph types in it
    // and make it convertible to yaml array.
    $config = [];

    foreach ($settings['handler_settings']['restrictions'] as $key => $restriction) {
      $allowed = [];
      foreach ($restriction as $name => $value) {
        if ($value !== 0) {
          $allowed[] = $name;
        }
      }
      $config[$key] = $allowed;
    }
    $settings['handler_settings']['restrictions'] = $config;
    return $settings;
  }

  /**
   * Manipulate configuration data for settings form.
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {

    $form = parent::fieldSettingsForm($form, $form_state);
    $handler_settings = $this->getSetting('handler_settings');

    $form['handler']['handler_settings']['restrictions'] = ['#prefix' => '<b>' . t('Restrictions:') . '</b>'];

    foreach ($handler_settings['allowed_layouts'] as $allowed_layout_group) {
      foreach ($allowed_layout_group as $key => $item) {
        $form['handler']['handler_settings']['restrictions'][$key] = [
          '#type' => 'checkboxes',
          '#options' => $form['handler']['handler_settings']['target_bundles']['#options'],
          '#title' => $item,
          '#default_value' => $handler_settings['restrictions'][$key],
        ];
      }
    }
    return $form;
  }

}
