<?php

namespace Drupal\erl_restriction\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\entity_reference_layout\Plugin\Field\FieldWidget\EntityReferenceLayoutWidget;

/**
 * Entity Reference with Layout field widget.
 *
 * @FieldWidget(
 *   id = "erl_restriction_widget",
 *   label = @Translation("Entity reference layout (With layout builder and restriction)"),
 *   description = @Translation("Layout builder for paragraphs."),
 *   field_types = {
 *     "erl_restriction"
 *   },
 * )
 */
class ErlRestrictionWidget extends EntityReferenceLayoutWidget implements ContainerFactoryPluginInterface {

  /**
   * Builds the main widget form array container/wrapper.
   *
   * Form elements for individual items are built by formElement().
   */
  protected function formMultipleElements(FieldItemListInterface $items, array &$form, FormStateInterface $form_state) {

    $elements = parent::formMultipleElements($items, $form, $form_state);

    $handler_settings = $items->getSetting('handler_settings');

    foreach ($elements as $key => $layout) {

      if (!is_numeric($key) || !isset($layout['layout']['#value']) || !isset($elements['add_more']['type']['#options'])) {
        continue;
      }

      // Allowed paragraphs in field.
      $option_keys = [];
      foreach ($elements['add_more']['type']['#options'] as $machine_name => $name) {
        $option_keys[] = $machine_name;
      }

      // Get the name of the layout (eg. layout_image_right).
      $layout = $layout['layout']['#value'];
      $intersect = array_intersect($handler_settings['restrictions'][$layout], $option_keys);

      $allowed = [];

      // Allowed in both, field and specific layout.
      foreach ($intersect as $machine_name) {
        $allowed[$machine_name] = $elements['add_more']['type']['#options'][$machine_name];
      }

      $elements[$key]['allowed'] = [
        '#type' => 'select',
        '#options' => $allowed,
        '#attributes' => [
          'class' => ['erl-item-type', 'hidden'],
          'id' => $elements["#field_name"] . '--item-' . $key . '-select',
        ],
      ];
    }
    return $elements;
  }

}
